/*
 * Copyright (c) 2014, 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package org.graalvm.compiler.truffle.compiler.nodes;

import static org.graalvm.compiler.nodeinfo.NodeCycles.CYCLES_1;

import org.graalvm.compiler.core.common.type.IntegerStamp;
import org.graalvm.compiler.core.common.type.StampFactory;
import org.graalvm.compiler.graph.IterableNodeType;
import org.graalvm.compiler.graph.NodeClass;
import org.graalvm.compiler.nodeinfo.NodeInfo;
import org.graalvm.compiler.nodeinfo.NodeSize;
import org.graalvm.compiler.nodes.NodeView;
import org.graalvm.compiler.nodes.ValueNode;
import org.graalvm.compiler.nodes.calc.FloatingNode;
import org.graalvm.compiler.nodes.calc.SignExtendNode;
import org.graalvm.compiler.nodes.calc.ZeroExtendNode;

import jdk.vm.ci.meta.JavaKind;

/**
 * The {@code AnyExtendNode} extends an i32 integer to an i64 integer, with the upper bits of the
 * result being undefined. This node itself cannot produce code, it needs to be converted to a
 * {@link ZeroExtendNode} or {@link SignExtendNode} explicitly.
 */
@NodeInfo(cycles = CYCLES_1, size = NodeSize.SIZE_1)
public final class AnyExtendNode extends FloatingNode implements IterableNodeType {

    public static final NodeClass<AnyExtendNode> TYPE = NodeClass.create(AnyExtendNode.class);

    public static final int INPUT_BITS = 32;
    public static final int OUTPUT_BITS = 64;

    @Input protected ValueNode value;

    public AnyExtendNode(ValueNode value) {
        super(TYPE, StampFactory.forKind(JavaKind.Long));
        assert value.stamp(NodeView.DEFAULT) instanceof IntegerStamp;
        this.value = value;
    }

    public ValueNode getValue() {
        return value;
    }
}
