/*
 * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package micro.benchmarks;

import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.State;

public class ObjectCloneArrayLengthBenchmark extends BenchmarkBase {

    static class A {
        final int x;

        A(int x) {
            this.x = x;
        }
    }

    @State(Scope.Benchmark)
    public static class ThreadState {
        int length = 10;
        A[] array = new A[]{new A(1), new A(2), new A(3), new A(4), new A(5)};
    }

    @Benchmark
    public int arrayAllocLength(ThreadState t) {
        return new int[t.length].length;
    }

    @Benchmark
    public int arrayAllocCloneLength(ThreadState t) {
        return new int[t.length].clone().length;
    }

    @Benchmark
    public int arrayLength(ThreadState t) {
        return t.array.length;
    }

    @Benchmark
    public int arrayCloneLength(ThreadState t) {
        return t.array.clone().length;
    }
}
